/* Copyright (c) 2007
   Colin Dewey (University of Wisconsin-Madison)
   cdewey@biostat.wisc.edu
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "deweylab/bio/alphabet/AlphabetCounter.hh"

#include <stdexcept>

namespace deweylab {
namespace bio {
namespace alphabet {

AlphabetCounter::AlphabetCounter(const bio::alphabet::Alphabet& alphabet)
	: alphabet(alphabet), counts(alphabet.getSize(), 0), total(0) {
}

void AlphabetCounter::count(const std::string& s) {
	count(s.begin(), s.end());
}

void AlphabetCounter::reset() {
	std::fill(counts.begin(), counts.end(), 0);
	total = 0;
}

void AlphabetCounter::addCounts(const AlphabetCounter& other) {
	if (&alphabet != &other.alphabet) {
		throw std::logic_error("Attempted to add counters with different alphabets");
	}
	for (size_t i = 0; i < counts.size(); ++i) {
		counts[i] += other.counts[i];
	}
	total += other.total;
}

}
}
}
