/* Copyright (c) 2007
   Colin Dewey (University of Wisconsin-Madison)
   cdewey@biostat.wisc.edu
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <string>
#include <algorithm>

#include "deweylab/bio/alphabet/AminoAcid.hh"
#include "deweylab/bio/alphabet/Alphabet.hh"
#include "deweylab/util/charmanip.hh"
#include "deweylab/util/stl.hh"

namespace deweylab {
namespace bio {
namespace alphabet {

void AminoAcid::hardMaskInPlace(std::string& seq) {
    std::transform(seq.begin(), seq.end(), seq.begin(),
				   util::stl::make_functor_ref(hardMasker));
}

std::string AminoAcid::hardMask(std::string seq) {
    hardMaskInPlace(seq);
    return seq;
}

void AminoAcid::unMaskInPlace(std::string& seq) {
    std::transform(seq.begin(), seq.end(), seq.begin(),
				   util::stl::make_functor_ref(unMasker));
}

std::string AminoAcid::unMask(std::string seq) {
    unMaskInPlace(seq);
    return seq;
}

const util::charmanip::Mapper
AminoAcid::hardMasker("abcdefghijklmnopqrstuvwxyz",
					  "XXXXXXXXXXXXXXXXXXXXXXXXXX");
const util::charmanip::Mapper
AminoAcid::unMasker("abcdefghijklmnopqrstuvwxyz",
					"ABCDEFGHIJKLMNOPQRSTUVWXYZ");
	
} } }
