/* Copyright (c) 2007
   Colin Dewey (University of Wisconsin-Madison)
   cdewey@biostat.wisc.edu
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <ostream>
#include "OutputStream.hh"

namespace deweylab {
namespace bio {
namespace formats {
namespace fastq {

OutputStream::OutputStream(std::ostream& stream,
                           const QualityScoreEncoding* encoding)
	: stream_(stream),
      qualities_encoding_(encoding),
	  line_width_(DEFAULT_LINE_WIDTH),
      repeat_title_(false)
{
}

void write_title_line(const Record& r,
                      std::ostream& stream,
                      const char prefix,
                      const bool include_title) {
	stream << prefix;
    if (include_title) {
        stream << r.id;
        if (not r.description.empty()) {
            stream << ' ' << r.description;
        }
    }
    stream << '\n';
}

void write_wrapped_string(const std::string& str,
                          std::ostream& stream,
                          const size_t line_width) {
    if (line_width == 0) { // Output unwrapped string
        stream << str << '\n';
    } else { // Output string wrapped at desired line width
        for (size_t start = 0; start < str.size(); start += line_width) {
            stream << str.substr(start, line_width) << '\n';
        }
    }
}

OutputStream& OutputStream::operator<<(const Record& r) {
    write_title_line(r, stream_, SEQUENCE_TITLE_LINE_PREFIX, true);
    write_wrapped_string(r.sequence, stream_, line_width_);
    write_title_line(r, stream_, QUALITIES_TITLE_LINE_PREFIX, repeat_title_);
    write_wrapped_string(qualities_encoding_->encode(r.qualities,
                                                     r.qualities_encoding),
                         stream_,
                         line_width_);
	return *this;
}

}
}
}
}
