/* Copyright (c) 2007
   Colin Dewey (University of Wisconsin-Madison)
   cdewey@biostat.wisc.edu
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __DEWEYLAB_BIO_FORMATS_FASTQ_RECORD_HH__
#define __DEWEYLAB_BIO_FORMATS_FASTQ_RECORD_HH__

#include <string>
#include "QualityScoreEncoding.hh"

namespace deweylab {
namespace bio {
namespace formats {
namespace fastq {

class Record {
public:
    std::string id;
    std::string description;
    std::string sequence;
    std::string qualities;
    const QualityScoreEncoding* qualities_encoding;

    PhredScore phred_quality(size_t i) const;
    SolexaScore solexa_quality(size_t i) const;

    void set_phred_qualities(const std::vector<PhredScore>& phred_scores);
    void set_solexa_qualities(const std::vector<SolexaScore>& solexa_scores);
};

inline PhredScore Record::phred_quality(size_t i) const {
    return qualities_encoding->decode_phred(qualities[i]);
}

inline SolexaScore Record::solexa_quality(size_t i) const {
    return qualities_encoding->decode_solexa(qualities[i]);
}

}
}
}
}

#endif // __DEWEYLAB_BIO_FORMATS_FASTQ_RECORD_HH__
