/* Copyright (c) 2007
   Colin Dewey (University of Wisconsin-Madison)
   cdewey@biostat.wisc.edu
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __DEWEYLAB_BIO_FORMATS_GFF_INPUTSTREAM_HH__
#define __DEWEYLAB_BIO_FORMATS_GFF_INPUTSTREAM_HH__

#include <string>
#include <iosfwd>

#include "deweylab/util/io/line/InputStream.hh"
#include "deweylab/bio/formats/gff/Record.hh"
#include "deweylab/util/stl.hh"

namespace deweylab {
namespace bio {
namespace formats {
namespace gff {

class InputStream {
public:
    typedef Record ValueType;
private:
    util::io::line::InputStream stream;
    std::string line;

public:
    InputStream(std::istream& stream,
				const size_t bufferSize=IO_BUFFER_SIZE);

    InputStream& operator>>(Record& rec);
    operator bool();
    bool operator!();
};

inline InputStream::InputStream(std::istream& stream,
								const size_t bufferSize) :
    stream(stream, bufferSize) {
}
	
inline InputStream::operator bool() {
    return stream;
}
	
inline bool InputStream::operator!() {
    return !stream;
}

typedef util::stl::InputStreamIterator<InputStream, Record> InputStreamIterator;

} } } }

#endif // __DEWEYLAB_BIO_FORMATS_GFF_INPUTSTREAM_HH__
