# Copyright (c) 2007
# Colin Dewey (University of Wisconsin-Madison)
# cdewey@biostat.wisc.edu
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
class MUM:
    def __init__(self,
                 ref, refStart, refEnd, refStrand,
                 query, queryStart, queryEnd, queryStrand):
        self.ref = ref
        self.refStart = refStart
        self.refEnd = refEnd
        self.refStrand = refStrand
        self.query = query
        self.queryStart = queryStart
        self.queryEnd = queryEnd
        self.queryStrand = queryStrand

    def __str__(self):
        return '\t'.join(map(str, (self.ref, self.refStart, self.refEnd,
                                   self.refStrand, self.query, self.queryStart,
                                   self.queryEnd, self.queryStrand)))

    def getRefInterval(self):
        return (self.ref, self.refStart, self.refEnd, self.refStrand)

    def getQueryInterval(self):
        return (self.query, self.queryStart, self.queryEnd, self.queryStrand)

    def getLength(self):
        return self.refEnd - self.refStart

class Iterator:
    REVERSE_STRING = " Reverse\n"
    
    def __init__(self, stream):
        self.stream = stream

    def __iter__(self):
        return self

    def next(self):
        while 1:
            line = self.stream.readline()
            if not line:
                raise StopIteration
            if line.startswith('>'):
                if line.endswith(self.REVERSE_STRING):
                    self.queryStrand = "-"
                    self.query = line[2:-len(self.REVERSE_STRING)]
                else:
                    self.queryStrand = "+"
                    self.query = line[2:-1]
            else:
                ref, refStart, queryStart, length = line.split()

                length = long(length)
                refStart = long(refStart) - 1
                refEnd = refStart + length

                if self.queryStrand == '-':
                    queryEnd = long(queryStart)
                    queryStart = queryEnd - length
                else:
                    queryStart = long(queryStart) - 1
                    queryEnd = queryStart + length

                return MUM(ref, refStart, refEnd, '+',
                           self.query, queryStart, queryEnd, self.queryStrand)
