/* Copyright (c) 2010
   Colin Dewey (University of Wisconsin-Madison)
   cdewey@biostat.wisc.edu
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <ostream>

#include "deweylab/bio/formats/qual/OutputStream.hh"

namespace deweylab {
namespace bio {
namespace formats {
namespace qual {

OutputStream::OutputStream(std::ostream& stream)
	: stream(stream),
	  lineWidth(DEFAULT_LINE_WIDTH) {
}
		
void OutputStream::setLineWidth(size_t lineWidth) {
	this->lineWidth = lineWidth;
}
	
OutputStream& OutputStream::operator<<(const Record& r) {
	// Output title line
	stream << TITLE_LINE_PREFIX << r.id << ' ' << r.description << '\n';

	// Output qualities wrapped at desired width
    size_t step = (lineWidth > 0 ? lineWidth : r.qualities.size());
	for (size_t start = 0; start < r.qualities.size(); start += step) {
        size_t end = std::min(start + step, r.qualities.size());
        for (size_t i = start; i < end; ++i) {
            if (i != start) { stream << " "; }
            stream << static_cast<unsigned int>( r.qualities[i]);
        }
        stream << "\n";
    }
	return *this;
}
		
} } } }
