# Copyright (c) 2007
# Colin Dewey (University of Wisconsin-Madison)
# cdewey@biostat.wisc.edu
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
from __future__ import generators

_FIELD_NAMES = ("score",
                "tName", "tSize", "tStrand", "tStart", "tEnd",
                "qName", "qSize", "qStrand", "qStart", "qEnd",
                "id")
_FIELD_TYPES = (int,
                str, int, str, int, int,
                str, int, str, int, int,
                int)

class Record:
    def __init__(self, alignments, **fields):
        self.__dict__["alignments"] = alignments
        self.__dict__["fields"] = fields

    def __getattr__(self, key):
        return self.fields[key]

    def __setattr__(self, key, value):
        self.fields[key] = value
        
    def __str__(self):
        s = "chain " + ' '.join([str(self.fields[key]) for key in _FIELD_NAMES])
        s += '\n'
        s += '\n'.join([' '.join(map(str, t)) for t in self.alignments])
        s += '\n'
        return s

def Iterator(stream):
    rec = None
    for line in stream:
        if line == "\n":
            yield rec
            rec = None
        elif line.startswith("#"):
            continue
        elif rec is None:
            fieldVals = [t(f) for t, f in zip(_FIELD_TYPES, line.split()[1:])]
            fields = dict(zip(_FIELD_NAMES, fieldVals))
            rec = Record([], **fields)
        else:
            rec.alignments.append(map(int, line.split()))
