/* Copyright (c) 2007
   Colin Dewey (University of Wisconsin-Madison)
   cdewey@biostat.wisc.edu
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "deweylab/bio/formats/fasta/DB.hh"
#include "deweylab/bio/formats/fasta/InputStream.hh"
#include "deweylab/filesystem.hh"

namespace deweylab {
namespace bio {
namespace formats {
namespace fasta {

DB::DB(const std::string& filename) {
	filesystem::InputFileStream stream(filename);
	read(stream);
}

DB::DB(std::istream& stream) {
	read(stream);
}

void DB::read(std::istream& stream) {
	InputStream rec_stream(stream);
	Record rec;
	while (rec_stream >> rec) {
		seqMap[rec.id] = rec.sequence;
	}	
}

std::string DB::getSeq(const std::string& id) {
	SeqMap::const_iterator it = seqMap.find(id);
	if (it == seqMap.end()) {
		throw std::runtime_error("Sequence not found: " + id);
	}
	return it->second;
}

} } } }
