/* Copyright (c) 2007
   Colin Dewey (University of Wisconsin-Madison)
   cdewey@biostat.wisc.edu
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "deweylab/app/FastaPipe.hh"
#include "deweylab/bio/alphabet/Nucleotide.hh"
#include "deweylab/bio/alphabet/AminoAcid.hh"
using namespace deweylab;
using namespace app;

class FastaHardmasker : public FastaPipe {
public:
    FastaHardmasker() {
        parser.setDescription("Hardmasks softmasked sequence");
        parser.addStoreTrueOpt('p', "protein",
                               "Input sequences are protein (default is DNA)",
                               protein = false);
    }
        
    bool process_record(Record& rec) {
        if (protein) {
            bio::alphabet::AminoAcid::hardMaskInPlace(rec.sequence);
        } else {
            bio::alphabet::Nucleotide::hardMaskInPlace(rec.sequence);
        }
        return true;
    }
    
private:
	bool protein;
};

int main(int argc, const char* argv[]) {
    FastaHardmasker app;
    return app.main(argc, argv);
}
