/* Copyright (c) 2007
   Colin Dewey (University of Wisconsin-Madison)
   cdewey@biostat.wisc.edu
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <iostream>

#include "deweylab/app/FastaReader.hh"
using namespace deweylab;
using namespace app;

class FastaLister : public FastaReader {
public:
    FastaLister();
    void run();
    void read_record(Record& rec);
protected:
    bool output_length;
    bool output_description;
    bool output_sequence;
    bool output_header;
};

FastaLister::FastaLister() {
    parser.setDescription("Outputs the id of each sequence in the input "
                          "FASTA file.  Optionally, the description, length, "
                          "or sequence may be output as additional fields.");
	parser.addStoreTrueOpt('d', "description",
						   "output sequence descriptions",
						   output_description = false);
	parser.addStoreTrueOpt('l', "length",
						   "output sequence lengths",
						   output_length = false);
	parser.addStoreTrueOpt('s', "sequence",
						   "output sequences",
						   output_sequence = false);
	parser.addStoreTrueOpt('h', "header",
						   "output header line with field names",
						   output_header = false);
}

void FastaLister::run() {
    if (output_header) {
        std::cout << "id";
        if (output_description) std::cout << '\t' << "description";
        if (output_length) std::cout << '\t' << "length";
        if (output_sequence) std::cout << '\t' << "sequence";
        std::cout << '\n';
    }
    FastaReader::run();
}

void FastaLister::read_record(Record& rec) {
    std::cout << rec.id;
    if (output_description) std::cout << '\t' << rec.description;
    if (output_length) std::cout << '\t' << rec.sequence.length();
    if (output_sequence) std::cout << '\t' << rec.sequence;
    std::cout << '\n';
}

int main(int argc, const char* argv[]) {
    FastaLister app;
    return app.main(argc, argv);
}
