#!/usr/bin/env python

# Copyright (c) 2007
# Colin Dewey (University of Wisconsin-Madison)
# cdewey@biostat.wisc.edu
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


from deweylab.bio.formats import fasta
import random
import sys
from optparse import OptionParser

usage = "usage: %prog [options] < faInput > faOutput"
parser = OptionParser(usage)
parser.add_option("-r", "--rate", type="float", dest="rate",
                  help="probability that a given base mutates",
                  default=0.1)
parser.add_option("-s", "--seed", type="int", dest="seed",
                  help="seed for initializing RNG",
                  default=None)
(options, args) = parser.parse_args()
if len(args) != 0:
    parser.error("incorrect number of arguments")

mutations = {'A': ('C', 'G', 'T'),
             'C': ('A', 'G', 'T'),
             'G': ('A', 'C', 'T'),
             'T': ('A', 'C', 'G')}

def mutateCharacter(char, rate):
    if random.random() < rate:
        return random.choice(mutations[char])
    else:
        return char

def mutateSequence(seq, rate):
    return ''.join(map(mutateCharacter, seq, [rate] * len(seq)))

random.seed(options.seed)

writer = fasta.Writer(sys.stdout)
for rec in fasta.Reader(sys.stdin):
    rec.sequence = mutateSequence(rec.sequence, options.rate)
    writer.write(rec)
