/* Copyright (c) 2007
   Colin Dewey (University of Wisconsin-Madison)
   cdewey@biostat.wisc.edu
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <iostream>
#include <vector>
#include <algorithm>
#include <ctime>

#include "boost/random/mersenne_twister.hpp"
#include "boost/random/random_number_generator.hpp"

#include "deweylab/bio/formats/fasta.hh"
#include "deweylab/util/options.hh"
using namespace deweylab;
using namespace bio::formats::fasta;

typedef std::vector<Record> RecordList;
typedef boost::mt19937 BaseGeneratorType;
typedef boost::random_number_generator<BaseGeneratorType, 
									   RecordList::difference_type> RNG;

int main(int argc, const char* argv[]) {
	// Increase speed of input/output to standard streams
	std::ios::sync_with_stdio(false);

	BaseGeneratorType::result_type seed = std::time(0);
	
	// Parse command line
	util::options::Parser parser("< fastaInput", 
			                     "Randomly shuffle FASTA records");
	parser.addStoreOpt('s', "seed", "Seed for random number generator",
					   seed, "INTEGER");	
	parser.parse(argv, argv + argc);

	try {
		InputStream in_stream(std::cin);
		InputStreamIterator in_begin(in_stream), in_end;
		OutputStream out_stream(std::cout);
		OutputStreamIterator out_begin(out_stream);		
		BaseGeneratorType generator(seed);
		RNG rng(generator);
		RecordList recs;
		
		std::copy(in_begin, in_end, std::back_inserter(recs));
		std::random_shuffle(recs.begin(), recs.end(), rng);
		std::copy(recs.begin(), recs.end(), out_begin);
		
	} catch (const std::runtime_error& e) {
		std::cerr << "Error: " << e.what() << '\n';
		return EXIT_FAILURE;
	}
		
	return EXIT_SUCCESS;
}
